from operator import attrgetter

class AVL:
    NO_VALUE = object()

    def __init__(self, dict_or_key, value = NO_VALUE):
        if value == AVL.NO_VALUE:
            k = next(iter(dict_or_key.keys()))
            v = dict_or_key.pop(k)
            self.key = k
            self.value = v
            self.left = None
            self.right = None
            self.height = 0

            for key, value in dict_or_key.items():
                self.set(key, value)
        else:
            self.key = dict_or_key
            self.value = value
            self.left = None
            self.right = None
            self.height = 0

    def recalculate_height(self):
        if self.left is None:
            left_height = 0
        else:
            self.left.recalculate_height()
            left_height = self.left.height + 1

        if self.right is None:
            right_height = 0
        else:
            self.right.recalculate_height()
            right_height = self.right.height + 1

        self.height = max(left_height, right_height)

    def make_deletable(self):
        parent = self
        if self.left is not None:
            target = self.left
            get_child = attrgetter('right')
        elif self.right is not None:
            target = self.right
            get_child = attrgetter('left')
        else:
            self._delete_from = None
            return

        while True:
            child = get_child(target)
            if child is None:
                break
            parent = target
            target = child

        self.value, target.value = target.value, self.value

        target._delete_from = parent
        return target

    def rotate_clockwise(self):
        moved = self.right
        self.right = moved.right
        moved.right = moved.left
        moved.left = self.left
        self.left = moved
        self.key, moved.key = moved.key, self.key
        self.value, moved.value = moved.value, self.value
        self.recalculate_height()

    def rotate_counterclockwise(self):
        moved = self.left
        self.left = moved.left
        moved.left = moved.right
        moved.right = self.right
        self.right = moved
        self.key, moved.key = moved.key, self.key
        self.value, moved.value = moved.value, self.value
        self.recalculate_height()

    def _locate(self, key, create_if_missing):
        if key < self.key:
            if self.left is None:
                if create_if_missing:
                    self.left = ret = AVL(key, None)
                    return [ret]
                raise KeyError('Key not found: %s' % key)
            return [self] + self.left._locate(key, create_if_missing)
        elif key > self.key:
            if self.right is None:
                if create_if_missing:
                    self.right = ret = AVL(key, None)
                    return [ret]
                raise KeyError('Key not found: %s' % key)
            return [self] + self.right._locate(key, create_if_missing)
        else:
            return [self]

    def locate(self, key):
        return self._locate(key, False)[-1]

    def get(self, key):
        return self.locate(key).value

    def _balance(self):
        left = self.left
        right = self.right

        if left:
            left_height = left.height + 1
        else:
            left_height = 0

        if right:
            right_height = right.height + 1
        else:
            right_height = 0

        return right_height - left_height

    def set(self, key, value):
        path = self._locate(key, True)
        self.recalculate_height()

        target = path.pop()
        target.value = value

        path.reverse()

        for node in path:
            balance = node._balance()

            if balance == 2:
                if node.right._balance() == -1:
                    node.right.rotate_counterclockwise()
                node.rotate_clockwise()
            elif balance == -2:
                if node.left._balance() == 1:
                    node.left.rotate_clockwise()
                node.rotate_counterclockwise()

    def remove(self, key):
        target = self.locate(key)
        target = target.make_deletable()

        if target is None:
            raise KeyError(key)

        parent = target._delete_from
        if parent:
            if parent.left is target:
                parent.left = None
            elif parent.right is target:
                parent.right = None
