from unittest import TestCase
from unittest.mock import patch, Mock
from planner.data import Status, TaskError
from datetime import datetime

class constructor_tests(TestCase):
    def test_valid(self):
        status = Status('status name',
                        datetime(year = 2012, month = 9, day = 11),
                        datetime(year = 2013, month = 4, day = 27))

        self.assertEqual(status.name, 'status name')
        self.assertEqual(status.begins,
                         datetime(year = 2012, month = 9, day = 11))
        self.assertEqual(status.ends,
                         datetime(year = 2013, month = 4, day = 27))

    def test_backwards_times(self):
        self.assertRaises(TaskError,
                          Status,
                          'status name',
                          datetime(year = 2013, month = 4, day = 27),
                          datetime(year = 2012, month = 9, day = 11))

    def test_too_short(self):
        self.assertRaises(TaskError,
                          Status,
                          'status name',
                          datetime(year = 2013, month = 4, day = 27,
                                   hour = 7, minute = 15),
                          datetime(year = 2013, month = 4, day = 27,
                                   hour = 7, minute = 15))

class utility_tests(TestCase):
    def test_repr(self):
        status = Status('status name',
                        datetime(year = 2012, month = 9, day = 11),
                        datetime(year = 2013, month = 4, day = 27))

        expected = "<status name 2012-09-11T00:00:00 2013-04-27T00:00:00>"

        self.assertEqual(repr(status), expected)

class exclusivity_tests(TestCase):
    def test_excludes(self):
        status = Mock()

        other = Status('status name',
                       datetime(year = 2012, month = 9, day = 11),
                       datetime(year = 2012, month = 10, day = 6))

        # A status shouldn't exclude anything
        self.assertFalse(Status.excludes(status, other))
        self.assertFalse(Status.excludes(status, None))

class overlap_tests(TestCase):
    def test_overlap_before(self):
        status = Mock(begins = datetime(year = 2012, month = 9, day = 11),
                      ends = datetime(year = 2012, month = 10, day = 6))

        other = Mock(begins = datetime(year = 2012, month = 10, day = 7),
                     ends = datetime(year = 2013, month = 2, day = 5))

        self.assertFalse(Status.overlaps(status, other))

    def test_overlap_begin(self):
        status = Mock(begins = datetime(year = 2012, month = 8, day = 11),
                      ends = datetime(year = 2012, month = 11, day = 27))

        other = Mock(begins = datetime(year = 2012, month = 10, day = 7),
                     ends = datetime(year = 2013, month = 2, day = 5))

        self.assertTrue(Status.overlaps(status, other))

    def test_overlap_end(self):
        status = Mock(begins = datetime(year = 2013, month = 1, day = 11),
                      ends = datetime(year = 2013, month = 4, day = 16))

        other = Mock(begins = datetime(year = 2012, month = 10, day = 7),
                     ends = datetime(year = 2013, month = 2, day = 5))

        self.assertTrue(Status.overlaps(status, other))

    def test_overlap_inner(self):
        status = Mock(begins = datetime(year = 2012, month = 10, day = 11),
                      ends = datetime(year = 2013, month = 1, day = 27))

        other = Mock(begins = datetime(year = 2012, month = 10, day = 7),
                     ends = datetime(year = 2013, month = 2, day = 5))

        self.assertTrue(Status.overlaps(status, other))

    def test_overlap_outer(self):
        status = Mock(begins = datetime(year = 2012, month = 8, day = 12),
                      ends = datetime(year = 2013, month = 3, day = 15))

        other = Mock(begins = datetime(year = 2012, month = 10, day = 7),
                     ends = datetime(year = 2013, month = 2, day = 5))

        self.assertTrue(Status.overlaps(status, other))

    def test_overlap_after(self):
        status = Mock(begins = datetime(year = 2013, month = 2, day = 6),
                      ends = datetime(year = 2013, month = 4, day = 27))

        other = Mock(begins = datetime(year = 2012, month = 10, day = 7),
                     ends = datetime(year = 2013, month = 2, day = 5))

        self.assertFalse(Status.overlaps(status, other))
